@@ -11,7 +11,7 @@ sub list_mounts
 local(@rv, @p, $_, $i); $i = 0;
 
 # List normal filesystem mounts
-open(FSTAB, $config{fstab_file});
+open(FSTAB, "<".$config{fstab_file});
 while(<FSTAB>) {
 	chop; s/#.*$//g;
 	if (!/\S/) { next; }
@@ -23,7 +23,7 @@ while(<FSTAB>) {
 close(FSTAB);
 
 # List automount points
-open(AUTOTAB, $config{autofs_file});
+open(AUTOTAB, "<".$config{autofs_file});
 while(<AUTOTAB>) {
 	chop; s/#.*$//g;
 	if (!/\S/ || /^[+\-]/) { next; }
@@ -72,7 +72,7 @@ return $len;
 sub delete_mount
 {
 local(@fstab, $i, $line, $_);
-open(FSTAB, $config{fstab_file});
+open(FSTAB, "<".$config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
 $i = 0;
@@ -89,7 +89,7 @@ foreach (@fstab) {
 	}
 &close_tempfile(FSTAB);
 
-open(AUTOTAB, $config{autofs_file});
+open(AUTOTAB, "<".$config{autofs_file});
 @autotab = <AUTOTAB>;
 close(AUTOTAB);
 &open_tempfile(AUTOTAB, "> $config{autofs_file}");
@@ -113,7 +113,7 @@ sub change_mount
 local(@fstab, @autotab, $i, $line, $fsck, $dir, $_);
 $i = 0;
 
-open(FSTAB, $config{fstab_file});
+open(FSTAB, "<".$config{fstab_file});
 @fstab = <FSTAB>;
 close(FSTAB);
 &open_tempfile(FSTAB, "> $config{fstab_file}");
@@ -135,7 +135,7 @@ foreach (@fstab) {
 	}
 &close_tempfile(FSTAB);
 
-open(AUTOTAB, $config{autofs_file});
+open(AUTOTAB, "<".$config{autofs_file});
 @autotab = <AUTOTAB>;
 close(AUTOTAB);
 &open_tempfile(AUTOTAB, "> $config{autofs_file}");
@@ -592,7 +592,7 @@ if ($_[0] eq "swap") {
 	print "<tr> <td><b>Priority</b></td>\n";
 	print "<td><select name=swap_pri>\n";
 	for ($i = 0; $i < 11; ++$i) {
-		printf "<option value=\"%s\" %s>%s\n",
+		printf "<option value=\"%s\" %s>%s</option>\n",
 			$i, $options{"pri"} == $i ? "selected" : "", $i;
 		}
 	print "</select></td> </tr>\n";
@@ -607,7 +607,7 @@ if ($_[0] eq "swapfs") {
 	if (!defined($options{backfstype})) { $options{backfstype} = "nfs"; }
 	foreach (&list_fstypes()) {
 		if ($_ eq "cachefs") { next; }
-		printf "<option value=\"$_\" %s>$_\n",
+		printf "<option value=\"$_\" %s>$_</option>\n",
 			$_ eq $options{backfstype} ? "selected" : "";
 		}
 	print "</select></td>\n";
@@ -632,10 +632,10 @@ if ($_[0] eq "swapfs") {
 
 	print "<tr> <td><b>Consistency check</b></td>\n";
 	print "<td><select name=cfs_con>\n";
-	print "<option value=1> Periodically\n";
-	printf "<option value=0 %s> Never\n",
+	print "<option value=1>Periodically</option>\n";
+	printf "<option value=0 %s>Never</option>\n",
 		defined($options{"noconst"}) ? "selected" : "";
-	printf "<option value=2 %s> On demand\n",
+	printf "<option value=2 %s>On demand</option>\n",
 		defined($options{"demandconst"}) ? "selected" : "";
 	print "</select></td>\n";
 
@@ -674,26 +674,29 @@ if ($_[0] eq "autofs") {
 sub check_location
 {
 if ($_[0] eq "nfs") {
-	local($out, $temp, $mout, $dirlist);
+	local($out, $temp, $mout, $dirlist, $qnfs_host, $qnfs_dir, $qtemp);
+	$qnfs_host = quotemeta($in{nfs_host});
 
-        # Use ping and showmount to see if the host exists and is up
-        if ($in{nfs_host} !~ /^\S+$/) {
-                &error("'$in{nfs_host}' is not a valid hostname");
-                }
-	&execute_command("ping -c 1 '$in{nfs_host}'", undef, \$out, \$out);
-        if ($out =~ /unknown host/i) {
-                &error("The host '$in{nfs_host}' does not exist");
-                }
-        elsif ($out =~ /100\% packet loss/) {
-                &error("The host '$in{nfs_host}' is down");
-                }
-	&execute_command("showmount -e '$in{nfs_host}'", undef, \$out, \$out);
-        if ($out =~ /Unable to receive/) {
-                &error("The host '$in{nfs_host}' does not support NFS");
-                }
-        elsif ($?) {
-                &error("Failed to get mount list : $out");
-                }
+	if ($config{'nfs_check'}) {
+		# Use ping and showmount to see if the host exists and is up
+		if ($in{nfs_host} !~ /^\S+$/) {
+			&error("'$in{nfs_host}' is not a valid hostname");
+			}
+		&execute_command("ping -c 1 $qnfs_host", undef, \$out, \$out);
+		if ($out =~ /unknown host/i) {
+			&error("The host '$in{nfs_host}' does not exist");
+			}
+		elsif ($out =~ /100\% packet loss/) {
+			&error("The host '$in{nfs_host}' is down");
+			}
+		&execute_command("showmount -e $qnfs_host", undef, \$out, \$out);
+		if ($out =~ /Unable to receive/) {
+			&error("The host '$in{nfs_host}' does not support NFS");
+			}
+		elsif ($?) {
+			&error("Failed to get mount list : $out");
+			}
+		}
 
         # Validate directory name
         foreach (split(/\n/, $out)) {
@@ -708,7 +711,9 @@ if ($_[0] eq "nfs") {
 	# Try a test mount to see if filesystem is available
 	$temp = &transname();
 	&make_dir($temp, 0755);
-	&execute_command("mount $in{nfs_host}:$in{nfs_dir} $temp",
+	$qnfs_dir = quotemeta($in{nfs_dir});
+	$qtemp = quotemeta($temp);
+	&execute_command("mount $qnfs_host:$qnfs_dir $qtemp",
 			 undef, \$mout, \$mout);
 	if ($mout =~ /No such file or directory/) {
 		&error("The directory '$in{nfs_dir}' does not exist on the ".
@@ -1139,8 +1144,10 @@ return @rv ? join(',' , @rv) : "-";
 # Attempt to create a swap file 
 sub create_swap
 {
-local($out);
-$out = &backquote_logged("mkfile $_[1]$_[2] $_[0] 2>&1");
+local($out, $qsize, $qfile);
+$qsize = quotemeta($_[1].$_[2]);
+$qfile = quotemeta($_[0]);
+$out = &backquote_logged("mkfile $qsize $qfile 2>&1");
 if ($?) {
 	&unlink_file($_[0]);
 	return "mkfile failed : $out";
