@@ -22,9 +22,10 @@ for(i=0; i<n; i++) {
 		mntlist[i].f_mntonname,
 		mntlist[i].f_mntfromname,
 		mntlist[i].f_fstypename,
-		expand_flags(mntlist[i].f_flag),
-		mntlist[i].f_flag);
+		expand_flags(mntlist[i].f_flags),
+		mntlist[i].f_flags);
 	}
+return 0;
 }
 
 char *expand_flags(int f)
